
import { Mixin, mix } from "../../mixwith.js";
import {CSSSelectorConstructionMixin} from "./css_selector_construction_mixin.js";
import {CSSElementSelector,
        CSSAttributeSelector,
        CSSContentSelector,
        CSSInputSelector,
        CSSRelationSelector,
        CSSStateSelector,
        CSSTagSelector,
        CSSTypeSelector} from "./css_element_selector.js";

import {CSSJoinElementWithParent} from "./join/css_join_element_with_parent.js";
import {CSSJoinElement} from "./join/css_join_element.js";
import {CSSJoinFollowingElement} from "./join/css_join_following_element.js";
import {CSSJoinFollowingImmediateElement} from "./join/css_join_following_immediate_element.js";
import {CSSJoinNestedElement} from "./join/css_join_nested_element.js";

/*
    A single selector.
*/
export class CSSSelector extends mix(Object).with(CSSSelectorConstructionMixin) {

  /*****************
  *  Constructors  *
  *****************/

  constructor( parent_css_constructor, primary_selector ) {
    super( parent_css_constructor );
    this._primarySelector = primary_selector ?
                            primary_selector :
                            new CSSElementSelector( this );
  }

  /********************
  *  Selector Output  *
  ********************/

  get css() {
    if ( ! this._css ) {
      this._css = '';
      if ( this._preJoinDescriptors ) {
        for ( let this_join_descriptor of this._preJoinDescriptors ) {
          this._css = this_join_descriptor.joinPre( this._css );
        }
      }
      this._css += this.primarySelector.css;
      if ( this._postJoinDescriptors ) {
        for ( let this_join_descriptor of this._postJoinDescriptors ) {
          this._css = this_join_descriptor.joinPost( this._css );
        }
      }
    }
    return this._css;
  }

  /***************
  *  Properties  *
  ***************/

  get primarySelector() {
    return this._primarySelector;
  }

  get preJoinDescriptors() {
    if ( ! this._preJoinDescriptors ) {
      this._preJoinDescriptors = [];
    }
    return this._preJoinDescriptors;
  }

  get postJoinDescriptors() {
    if ( ! this._postJoinDescriptors ) {
      this._postJoinDescriptors = [];
    }
    return this._postJoinDescriptors;
  }

  /*
      element,element	div, p	Selects all <div> elements and all <p> elements
  */
  addElementSelector( element_selector ) {
    if ( ! element_selector ) {
      element_selector = new CSSElementSelector();
    }
    let additional_element_join = new CSSJoinElement(element_selector);
    this.postJoinDescriptors.push( additional_element_join );
    return element_selector;
  }

  addElementPreSelector( element_selector ) {
    if ( ! element_selector ) {
      element_selector = new CSSElementSelector();
    }
    let additional_element_join = new CSSJoinElement(element_selector);
    this.preJoinDescriptors.push( additional_element_join );
    return element_selector;
  }

  /*
      element element	div p	Selects all <p> elements inside <div> elements
  */
  addNestedElementSelector( nested_element_selector ) {
    if ( ! nested_element_selector ) {
      nested_element_selector = new CSSElementSelector();
    }
    let nested_element_join = new CSSJoinNestedElement(nested_element_selector);
    this.postJoinDescriptors.push( nested_element_join );
    return nested_element_selector;
  }

  addNestedElementPreSelector( nested_element_selector ) {
    if ( ! nested_element_selector ) {
      nested_element_selector = new CSSElementSelector();
    }
    let nested_element_join = new CSSJoinNestedElement(nested_element_selector);
    this.preJoinDescriptors.push( nested_element_join );
    return nested_element_selector;
  }

  /*
      element>element	div > p	Selects all <p> elements where the parent is a <div> element
  */
  addElementWithParentSelector( element_with_parent_selector ) {
    if ( ! element_with_parent_selector ) {
      element_with_parent_selector = new CSSElementSelector();
    }
    let element_with_parent_join = new CSSJoinElementWithParent(element_with_parent_selector);
    this.postJoinDescriptors.push( element_with_parent_join );
    return element_with_parent_selector;
  }

  addElementWithParentPreSelector( element_with_parent_selector ) {
    if ( ! element_with_parent_selector ) {
      element_with_parent_selector = new CSSElementSelector();
    }
    let element_with_parent_join = new CSSJoinElementWithParent(element_with_parent_selector);
    this.preJoinDescriptors.push( element_with_parent_join );
    return element_with_parent_selector;
  }

  /*
      element1~element2	p ~ ul	Selects every <ul> element that are preceded by a <p> element
  */
  addFollowingElementSelector( following_element_selector ) {
    if ( ! following_element_selector ) {
      following_element_selector = new CSSElementSelector();
    }
    let following_element_join = new CSSJoinFollowingElement(following_element_selector);
    this.postJoinDescriptors.push( following_element_join );
    return following_element_selector;
  }

  addFollowingElementPreSelector( following_element_selector ) {
    if ( ! following_element_selector ) {
      following_element_selector = new CSSElementSelector();
    }
    let following_element_join = new CSSJoinFollowingElement(following_element_selector);
    this.preJoinDescriptors.push( following_element_join );
    return following_element_selector;
  }

  /*
      element+element	div + p	Selects all <p> elements that are placed immediately after <div> elements
  */
  addFollowingImmediateElementSelector( immediate_following_element_selector ) {
    if ( ! immediate_following_element_selector ) {
      immediate_following_element_selector = new CSSElementSelector();
    }
    let immediate_following_element_join = new CSSJoinFollowingImmediateElement(immediate_following_element_selector);
    this.postJoinDescriptors.push( immediate_following_element_join );
    return immediate_following_element_selector;
  }

  addFollowingImmediateElementPreSelector( immediate_following_element_selector ) {
    if ( ! immediate_following_element_selector ) {
      immediate_following_element_selector = new CSSElementSelector();
    }
    let immediate_following_element_join = new CSSJoinFollowingImmediateElement(immediate_following_element_selector);
    this.preJoinDescriptors.push( immediate_following_element_join );
    return immediate_following_element_selector;
  }

  /*
      CSSSelector is treated as relay for primary CSSElementSelector
      for all non-conflicting interfaces.
  */

  set host( true_or_false ) {
    return this.primarySelector.host = true_or_false;
  }

  get host() {
    return this.primarySelector.host;
  }

  set slotted( true_or_false ) {
    return this.primarySelector.slotted = true_or_false;
  }

  get slotted() {
    return this.primarySelector.slotted;
  }

  set not( true_or_false ) {
    return this.primarySelector.not = true_or_false;
  }

  get not() {
    return this.primarySelector.not;
  }

  get tagSelector() {
    return this.primarySelector.tagSelector;
  }

  get notTagSelector() {
    return this.primarySelector.notTagSelector;
  }

  get attributeSelectors() {
    return this.primarySelector.attributeSelectors;
  }

  attributeSelector( attribute_name ) {
    return this.primarySelector.attributeSelector( attribute_name );
  }

  addAttributeSelector( attribute_name ) {
    return this.primarySelector.addAttributeSelector( attribute_name );
  }

  ensureAttributeSelector( attribute_name ) {
    return this.primarySelector.ensureAttributeSelector( attribute_name );
  }

  get notAttributeSelectors() {
    return this.primarySelector.notAttributeSelectors;
  }

  notAttributeSelector( attribute_name ) {
    return this.primarySelector.notAttributeSelector( attribute_name );
  }

  addNotAttributeSelector( attribute_name ) {
    return this.primarySelector.addNotAttributeSelector( attribute_name );
  }

  ensureNotAttributeSelector( attribute_name ) {
    return this.primarySelector.ensureNotAttributeSelector( attribute_name );
  }

  get inputSelector() {
    return this.primarySelector.inputSelector;
  }

  get notInputSelector() {
    return this.primarySelector.notInputSelector;
  }

  get stateSelector() {
    return this.primarySelector.stateSelector;
  }

  get notStateSelector() {
    return this.primarySelector.notStateSelector;
  }

  get typeSelector() {
    return this.primarySelector.typeSelector;
  }

  get notTypeSelector() {
    return this.primarySelector.notTypeSelector;
  }

  get relationSelector() {
    return this.primarySelector.relationSelector;
  }

  get notRelationSelector() {
    return this.primarySelector.notRelationSelector;
  }

  get contentSelector() {
    return this.primarySelector.contentSelector;
  }

  get notContentSelector() {
    return this.primarySelector.notContentSelector;
  }

  clearCSS() {
    super.clearCSS();
  }

}

export {CSSElementSelector,
        CSSAttributeSelector,
        CSSContentSelector,
        CSSInputSelector,
        CSSRelationSelector,
        CSSStateSelector,
        CSSTagSelector,
        CSSTypeSelector};
export default CSSSelector;
