
import { Mixin, mix } from "../../../mixwith.js";
import {CSSSelectorConstructionMixin} from "../css_selector_construction_mixin.js";

/*
    A single selector for attribute qualifiers.
*/
export class CSSAttributeSelector extends mix(Object).with(CSSSelectorConstructionMixin) {

  /*****************
  *  Constructors  *
  *****************/

  constructor( attribute_name, parent_selector = undefined ) {
    super( parent_selector );
    this._attributeName = attribute_name;
    this._shouldSelectAttributeName = false;
  }

  /********************
  *  Selector Output  *
  ********************/

  get css() {
    if ( ! this._css ) {
      if ( this.isValue ) {
        this._css = this.isValueSelector;
      }
      else {
        this._css = '';
        if ( this.beginsWithValue ) {
          this._css += this.beginsWithValueSelector;
        }
        else if ( this.beginsWithDashedWord ) {
          this._css += this.beginsWithDashedWordSelector;
        }
        if ( this._containsValues && this._containsValues.length ) {
          this._css += this.containsValuesSelector;
        }
        if ( this._containsSpacedWords && this._containsSpacedWords.length ) {
          this._css += this.containsSpacedWordsSelector;
        }
        if ( this.endsWithValue ) {
          this._css += this.endsWithValueSelector;
        }
        if ( ! this._css.length && this.shouldSelectAttributeName ) {
          this._css = this.attributeNameSelector;
        }
      }

    }
    return this._css;
  }

  /***************
  *  Properties  *
  ***************/

  /*
      * tag
      [attribute]	[target]	Selects all elements with a target attribute
  */

  set attributeName( selector_name ) {
    this.clearCSS();
    return this._attributeName = selector_name;
  }

  get attributeName() {
    return this._attributeName;
  }

  set shouldSelectAttributeName( true_or_false ) {
    this.clearCSS();
    return this._shouldSelectAttributeName = true_or_false;
  }

  get shouldSelectAttributeName() {
    return this._shouldSelectAttributeName;
  }

  get attributeNameSelector() {
    if ( ! this._attributeNameSelector ) {
      this._attributeNameSelector = '[' + this.attributeName + ']';
    }
    return this._attributeNameSelector;
  }

  /*
      * tag = value
      [attribute=value]	[target=_blank]	Selects all elements with target="_blank"
  */
  set isValue( attribute_value ) {
    this.clearCSS();
    return this._isValue = attribute_value;
  }

  get isValue() {
    return this._isValue;
  }

  get isValueSelector() {
    if ( ! this._isValueSelector ) {
      this._isValueSelector = '[' + this.attributeName + '="' + this.isValue + '"]';
    }
    return this._isValueSelector;
  }

  /*
      * tag ^= begins with value
      [attribute^=value]	a[href^="https"]	Selects every <a> element whose href attribute value begins with "https"
  */
  set beginsWithValue( attribute_value ) {
    this.clearCSS();
    return this._beginsWithValue = attribute_value;
  }

  get beginsWithValue() {
    return this._beginsWithValue;
  }

  get beginsWithValueSelector() {
    if ( ! this._beginsWithValueSelector ) {
      this._beginsWithValueSelector = '[' + this.attributeName + '^="' + this.beginsWithValue + '"]';
    }
    return this._beginsWithValueSelector;
  }

  /*
      * tag $= ends with value
      [attribute$=value]	a[href$=".pdf"]	Selects every <a> element whose href attribute value ends with ".pdf"
  */
  set endsWithValue( attribute_value ) {
    this.clearCSS();
    return this._endsWithValue = attribute_value;
  }

  get endsWithValue() {
    return this._endsWithValue;
  }

  get endsWithValueSelector() {
    if ( ! this._endsWithValueSelector ) {
      this._endsWithValueSelector = '[' + this.attributeName + '$="' + this.endsWithValue + '"]';
    }
    return this._endsWithValueSelector;
  }

  /*
      * tag *= contains value
      [attribute*=value]	a[href*="w3schools"]	Selects every <a> element whose href attribute value contains the substring "w3schools"
  */
  set containsValues( values_array ) {
    this.clearCSS();
    return this._containsValues = values_array;
  }

  get containsValues() {
    if ( ! this._containsValues ) {
      this._containsValues = [];
    }
    return this._containsValues;
  }

  addContainsValues( ...values_array ) {
    this.clearCSS();
    return this.containsValues.push( ...values_array );
  }

  get containsValuesSelector() {
    if ( ! this._containsValuesSelector ) {
      this._containsValuesSelector = '';
      for ( let this_value of this.containsValues ) {
        this._containsValuesSelector += '[' + this.attributeName + '*="' + this_value + '"]';
      }
    }
    return this._containsValuesSelector;
  }

  /*
      * tag ~= contains whole word optionally separated by spaces
      [attribute~=value]	[title~=flower]	Selects all elements with a title attribute containing the word "flower"
  */
  set containsSpacedWords( words_array ) {
    this.clearCSS();
    return this._containsSpacedWords = attribute_values;
  }

  addContainsSpacedWords( ...words_array ) {
    this.clearCSS();
    return this.containsSpacedWords.push( ...words_array );
  }

  get containsSpacedWords() {
    if ( ! this._containsSpacedWords ) {
      this._containsSpacedWords = [];
    }
    return this._containsSpacedWords;
  }

  get containsSpacedWordsSelector() {
    if ( ! this._containsSpacedWordsSelector ) {
      this._containsSpacedWordsSelector = '';
      for ( let this_value of this.containsSpacedWords ) {
        this._containsSpacedWordsSelector += '[' + this.attributeName + '~="' + this_value + '"]';
      }
    }
    return this._containsSpacedWordsSelector;
  }

  /*
      * tag |= starting whole world followed by optional hyphen
  */
  set beginsWithDashedWord( starting_word ) {
    this.clearCSS();
    return this._beginsWithDashedWord = starting_word;
  }

  get beginsWithDashedWord() {
    return this._beginsWithDashedWord;
  }

  get beginsWithDashedWordSelector() {
    if ( ! this._beginsWithDashedWordSelector ) {
      this._beginsWithDashedWordSelector = '[' + this.attributeName + '|="' + this.beginsWithDashedWord + '"]';
    }
    return this._beginsWithDashedWordSelector;
  }

  clearCSS() {
    super.clearCSS();
    delete this._attributeNameSelector
    delete this._isValueSelector
    delete this._beginsWithValueSelector
    delete this._endsWithValueSelector
    delete this._containsValuesSelector
    delete this._containsSpacedWordsSelector
    delete this._beginsWithDashedWordSelector
  }

}

export default CSSAttributeSelector;
